@@ -1,5 +1,5 @@
-/*  Lzip - Data compressor based on the LZMA algorithm
-    Copyright (C) 2008, 2009, 2010 Antonio Diaz Diaz.
+/*  Lzip - LZMA lossless data compressor
+    Copyright (C) 2008, 2009, 2010, 2011, 2012, 2013 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 class Range_decoder
   {
   enum { buffer_size = 16384 };
-  long long partial_member_pos;
+  unsigned long long partial_member_pos;
   uint8_t * const buffer;	// input buffer
   int pos;			// current pos in buffer
   int stream_pos;		// when reached, a new block must be read
@@ -29,8 +29,11 @@ class Range_decoder
 
   bool read_block();
 
+  Range_decoder( const Range_decoder & );	// declared as private
+  void operator=( const Range_decoder & );	// declared as private
+
 public:
-  Range_decoder( const int ifd )
+  explicit Range_decoder( const int ifd )
     :
     partial_member_pos( 0 ),
     buffer( new uint8_t[buffer_size] ),
@@ -39,28 +42,40 @@ public:
     code( 0 ),
     range( 0xFFFFFFFFU ),
     infd( ifd ),
-    at_stream_end( false ) {}
+    at_stream_end( false )
+    {}
 
   ~Range_decoder() { delete[] buffer; }
 
-  bool code_is_zero() const throw() { return ( code == 0 ); }
+  bool code_is_zero() const { return ( code == 0 ); }
   bool finished() { return pos >= stream_pos && !read_block(); }
-  long long member_position() const throw()
-    { return partial_member_pos + pos; }
-  void reset_member_position() throw()
-    { partial_member_pos = -pos; }
+  unsigned long long member_position() const { return partial_member_pos + pos; }
+  void reset_member_position() { partial_member_pos = -pos; }
 
   uint8_t get_byte()
     {
-    if( finished() ) return 0;
+    if( finished() ) return 0xAA;		// make code != 0
     return buffer[pos++];
     }
 
+  int read_data( uint8_t * const outbuf, const int size )
+    {
+    int rest = size;
+    while( rest > 0 && !finished() )
+      {
+      const int rd = std::min( rest, stream_pos - pos );
+      std::memcpy( outbuf + size - rest, buffer + pos, rd );
+      pos += rd;
+      rest -= rd;
+      }
+    return size - rest;
+    }
+
   void load()
     {
     code = 0;
-    range = 0xFFFFFFFFU;
     for( int i = 0; i < 5; ++i ) code = (code << 8) | get_byte();
+    range = 0xFFFFFFFFU;
     }
 
   void normalize()
@@ -74,17 +89,14 @@ public:
     int symbol = 0;
     for( int i = num_bits; i > 0; --i )
       {
-      symbol <<= 1;
-      if( range <= 0x00FFFFFFU )
-        {
-        range <<= 7; code = (code << 8) | get_byte();
-        if( code >= range ) { code -= range; symbol |= 1; }
-        }
-      else
-        {
-        range >>= 1;
-        if( code >= range ) { code -= range; symbol |= 1; }
-        }
+      normalize();
+      range >>= 1;
+//      symbol <<= 1;
+//      if( code >= range ) { code -= range; symbol |= 1; }
+      const uint32_t mask = 0U - (code < range);
+      code -= range;
+      code += range & mask;
+      symbol = (symbol << 1) + (mask + 1);
       }
     return symbol;
     }
@@ -110,10 +122,22 @@ public:
 
   int decode_tree( Bit_model bm[], const int num_bits )
     {
-    int model = 1;
+    int symbol = 1;
     for( int i = num_bits; i > 0; --i )
-      model = ( model << 1 ) | decode_bit( bm[model] );
-    return model - (1 << num_bits);
+      symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
+    return symbol - (1 << num_bits);
+    }
+
+  int decode_tree6( Bit_model bm[] )
+    {
+    int symbol = 1;
+    symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
+    symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
+    symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
+    symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
+    symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
+    symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
+    return symbol - (1 << 6);
     }
 
   int decode_tree_reversed( Bit_model bm[], const int num_bits )
@@ -122,77 +146,64 @@ public:
     int symbol = 0;
     for( int i = 0; i < num_bits; ++i )
       {
-      const int bit = decode_bit( bm[model] );
+      const bool bit = decode_bit( bm[model] );
       model <<= 1;
-      if( bit ) { model |= 1; symbol |= (1 << i); }
+      if( bit ) { ++model; symbol |= (1 << i); }
       }
     return symbol;
     }
 
-  int decode_matched( Bit_model bm[], const int match_byte )
+  int decode_tree_reversed4( Bit_model bm[] )
+    {
+    int model = 1;
+    int symbol = 0;
+    int bit = decode_bit( bm[model] );
+    model = (model << 1) + bit; symbol |= bit;
+    bit = decode_bit( bm[model] );
+    model = (model << 1) + bit; symbol |= (bit << 1);
+    bit = decode_bit( bm[model] );
+    model = (model << 1) + bit; symbol |= (bit << 2);
+    if( decode_bit( bm[model] ) ) symbol |= 8;
+    return symbol;
+    }
+
+  int decode_matched( Bit_model bm[], int match_byte )
     {
     Bit_model * const bm1 = bm + 0x100;
     int symbol = 1;
     for( int i = 7; i >= 0; --i )
       {
-      const int match_bit = ( match_byte >> i ) & 1;
-      const int bit = decode_bit( bm1[(match_bit<<8)+symbol] );
+      match_byte <<= 1;
+      const int match_bit = match_byte & 0x100;
+      const int bit = decode_bit( bm1[match_bit+symbol] );
       symbol = ( symbol << 1 ) | bit;
-      if( match_bit != bit )
+      if( match_bit != bit << 8 )
         {
-        while( --i >= 0 )
+        while( symbol < 0x100 )
           symbol = ( symbol << 1 ) | decode_bit( bm[symbol] );
         break;
         }
       }
-    return symbol & 0xFF;
+    return symbol - 0x100;
     }
-  };
 
-
-class Len_decoder
-  {
-  Bit_model choice1;
-  Bit_model choice2;
-  Bit_model bm_low[pos_states][len_low_symbols];
-  Bit_model bm_mid[pos_states][len_mid_symbols];
-  Bit_model bm_high[len_high_symbols];
-
-public:
-  int decode( Range_decoder & range_decoder, const int pos_state )
+  int decode_len( Len_model & lm, const int pos_state )
     {
-    if( range_decoder.decode_bit( choice1 ) == 0 )
-      return range_decoder.decode_tree( bm_low[pos_state], len_low_bits );
-    if( range_decoder.decode_bit( choice2 ) == 0 )
+    if( decode_bit( lm.choice1 ) == 0 )
+      return decode_tree( lm.bm_low[pos_state], len_low_bits );
+    if( decode_bit( lm.choice2 ) == 0 )
       return len_low_symbols +
-             range_decoder.decode_tree( bm_mid[pos_state], len_mid_bits );
+             decode_tree( lm.bm_mid[pos_state], len_mid_bits );
     return len_low_symbols + len_mid_symbols +
-           range_decoder.decode_tree( bm_high, len_high_bits );
+           decode_tree( lm.bm_high, len_high_bits );
     }
   };
 
 
-class Literal_decoder
-  {
-  Bit_model bm_literal[1<<literal_context_bits][0x300];
-
-  int lstate( const int prev_byte ) const throw()
-    { return ( prev_byte >> ( 8 - literal_context_bits ) ); }
-
-public:
-  uint8_t decode( Range_decoder & range_decoder, const uint8_t prev_byte )
-    { return range_decoder.decode_tree( bm_literal[lstate(prev_byte)], 8 ); }
-
-  uint8_t decode_matched( Range_decoder & range_decoder,
-                          const uint8_t prev_byte, const uint8_t match_byte )
-    { return range_decoder.decode_matched( bm_literal[lstate(prev_byte)],
-                                           match_byte ); }
-  };
-
-
 class LZ_decoder
   {
-  long long partial_data_pos;
+  unsigned long long partial_data_pos;
+  Range_decoder & rdec;
   const int dictionary_size;
   const int buffer_size;
   uint8_t * const buffer;	// output buffer
@@ -201,18 +212,17 @@ class LZ_decoder
   uint32_t crc_;
   const int outfd;		// output file descriptor
   const int member_version;
-  Range_decoder & range_decoder;
 
   void flush_data();
   bool verify_trailer( const Pretty_print & pp ) const;
 
-  uint8_t get_prev_byte() const throw()
+  uint8_t get_prev_byte() const
     {
     const int i = ( ( pos > 0 ) ? pos : buffer_size ) - 1;
     return buffer[i];
     }
 
-  uint8_t get_byte( const int distance ) const throw()
+  uint8_t get_byte( const int distance ) const
     {
     int i = pos - distance - 1;
     if( i < 0 ) i += buffer_size;
@@ -231,7 +241,7 @@ class LZ_decoder
     if( i < 0 ) i += buffer_size;
     if( len < buffer_size - std::max( pos, i ) && len <= std::abs( pos - i ) )
       {
-      std::memcpy( buffer + pos, buffer + i, len );
+      std::memcpy( buffer + pos, buffer + i, len );	// no wrap, no overlap
       pos += len;
       }
     else for( ; len > 0; --len )
@@ -242,10 +252,14 @@ class LZ_decoder
       }
     }
 
+  LZ_decoder( const LZ_decoder & );		// declared as private
+  void operator=( const LZ_decoder & );		// declared as private
+
 public:
-  LZ_decoder( const File_header & header, Range_decoder & rdec, const int ofd )
+  LZ_decoder( const File_header & header, Range_decoder & rde, const int ofd )
     :
     partial_data_pos( 0 ),
+    rdec( rde ),
     dictionary_size( header.dictionary_size() ),
     buffer_size( std::max( 65536, dictionary_size ) ),
     buffer( new uint8_t[buffer_size] ),
@@ -253,16 +267,13 @@ public:
     stream_pos( 0 ),
     crc_( 0xFFFFFFFFU ),
     outfd( ofd ),
-    member_version( header.version() ),
-    range_decoder( rdec )
+    member_version( header.version() )
     { buffer[buffer_size-1] = 0; }	// prev_byte of first_byte
 
   ~LZ_decoder() { delete[] buffer; }
 
-  uint32_t crc() const throw() { return crc_ ^ 0xFFFFFFFFU; }
-
-  long long data_position() const throw()
-    { return partial_data_pos + pos; }
+  unsigned crc() const { return crc_ ^ 0xFFFFFFFFU; }
+  unsigned long long data_position() const { return partial_data_pos + pos; }
 
   int decode_member( const Pretty_print & pp );
   };
